/*

    Zelda Navi's Quest

    Copyright (C) 2013-2014  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#include <iostream>
#include <cstdlib>
#include <ctime>

#include "engine/window/WindowManager.h"
#include "engine/audio/AudioManager.h"

#include "config/ConfigurationManager.h"

#include "game/MainController.h"

#include "game/content/helper/ItemHelper.h"

#include <pspdisplay.h>
#include <psppower.h>
#include <pspdebug.h>
#include <pspmoduleinfo.h>
#include <pspthreadman.h>
#include <psploadexec.h>
#include <pspctrl.h>

#define printf pspDebugScreenPrintf

PSP_MODULE_INFO("ZELDANSQ", PSP_MODULE_USER, 1, 2);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);
//PSP_HEAP_SIZE_KB(-128);
//PSP_MAIN_THREAD_STACK_SIZE_KB(128);
PSP_HEAP_SIZE_KB(-64);
PSP_MAIN_THREAD_STACK_SIZE_KB(192);

bool gLoop;
bool widescreen = false;

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common) {
    gLoop = false;
	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp) {
	int cbid;
	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);
	sceKernelSleepThreadCB();
	return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void) {
	int thid = 0;
	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0) {
		sceKernelStartThread(thid, 0, 0);
	}
	return thid;
}

int main(int argc, char** argv) {
    if (argc && argv); //pour viter un warning.....

	gLoop = true;

	scePowerSetClockFrequency(333,333,166); // This is needed when there is more than one enemy on-screen.
	SetupCallbacks();


	pspDebugScreenInit();
	pspDebugScreenSetTextColor(0xFF9900);


	pspDebugScreenSetXY(19, 11);
	printf("**** Zelda - Navi's Quest ****");
	pspDebugScreenSetXY(19, 14);
	printf("Press X for Normal Screen Mode");
	pspDebugScreenSetXY(20, 16);
	printf("Press O for Wide Screen Mode");

	SceCtrlData ctrl;
	sceCtrlPeekBufferPositive(&ctrl, 1);
	while(!(ctrl.Buttons & PSP_CTRL_CROSS) && !(ctrl.Buttons & PSP_CTRL_CIRCLE)){
		sceCtrlPeekBufferPositive(&ctrl, 1);
	}
	if (ctrl.Buttons & PSP_CTRL_CIRCLE) widescreen = true;
	pspDebugScreenSetXY(20, 20);
	printf("        Loading...");
	sceKernelDelayThread(2 * 1000 * 1000);
/*
    // Loading Screen.
    SDL_Rect srcI, dstI;
    srcI.w=200; srcI.h=64;
    srcI.y=0;srcI.x=0;dstI.x=(Screen_Width-200) >> 1; dstI.y=(Screen_Height-64) >> 1;
    SDL_Surface *gpLoading = IMG_Load("Loading.png");
	SDL_BlitSurface(gpLoading, &srcI, gpScreen, &dstI);
	SDL_Flip(gpScreen);
	SDL_FreeSurface(gpLoading);
*/

    std::srand(std::time(NULL));

    if (WindowManager::getInstance()->createWindow("Zelda Navi's Quest", "data/images/logos/navi.ico") < 0) {
        exit(-1);
    }

    ConfigurationManager::getInstance()->init("config/system.dat", "config/keys.dat");

    MainController* controller = MainController::getInstance();

    controller->init();

    controller->setInternalStep(LOGO);

    Event* event;

    // PSP additions
    sceCtrlSetSamplingCycle(0);
    sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

    while(WindowManager::getInstance()->isOpened() && gLoop == true) {

        event = WindowManager::getInstance()->getEvent();

        controller->control(event);

        if (event->isPushed(QUIT) || event->isPushed(kEscape) || event->isPushed(QUIT_FORCED)) {

            WindowManager::getInstance()->close();
            continue;
        }

        WindowManager::getInstance()->display();
    }

    ItemHelper::getInstance()->close();

    controller->close();

    ConfigurationManager::getInstance()->close();
    AudioManager::getInstance()->close();

    WindowManager::getInstance()->exit();

	sceKernelExitGame();	

    return 0;
}
